package org.sevensource.commons.jdbcconfiguration.database;

import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import javax.sql.DataSource;

import org.apache.commons.configuration.DatabaseConfiguration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class CachingDatabaseConfiguration extends DatabaseConfiguration {

	
	private final static Logger logger = LoggerFactory.getLogger(CachingDatabaseConfiguration.class);
	
	
	private ConcurrentMap<String, Object> databaseConfigurationCache = new ConcurrentHashMap<String, Object>();
	
	public CachingDatabaseConfiguration(DataSource datasource, String table,
			String nameColumn, String keyColumn, String valueColumn,
			String name, boolean commits) {
		super(datasource, table, nameColumn, keyColumn, valueColumn, name, commits);
	}
	
	@Override
	public Object getProperty(String key) {
		if(! databaseConfigurationCache.containsKey(key)) {
			Object property = super.getProperty(key);
			databaseConfigurationCache.put(key, property);
		}
		
		return databaseConfigurationCache.get(key);
	}
	
	@Override
	protected void addPropertyDirect(String key, Object obj) {	
		super.addPropertyDirect(key, obj);
		databaseConfigurationCache.remove(key);
	}
	
	@Override
	public void addProperty(String key, Object value) {
		super.addProperty(key, value);
		databaseConfigurationCache.remove(key);
	}
	
	@Override
	protected void clearPropertyDirect(String key) {
		super.clearPropertyDirect(key);
		databaseConfigurationCache.remove(key);
	}
	
	@Override
	public void clear() {
		super.clear();
		databaseConfigurationCache.clear();
	}
	
	public void clearCache() {
		databaseConfigurationCache.clear();
	}
	
}
