package org.sevensource.commons.jdbcconfiguration.database;

import org.apache.commons.configuration.CompositeConfiguration;
import org.apache.commons.configuration.Configuration;
import org.apache.commons.lang.NotImplementedException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ExtendedCompositeConfiguration extends CompositeConfiguration {

	private final static Logger logger = LoggerFactory.getLogger(ExtendedCompositeConfiguration.class);
	
	private Configuration serverConfiguration;
	private Configuration defaultConfiguration;
	
	public void setServerConfiguration(Configuration configuration) {
		this.serverConfiguration = configuration;
		addConfiguration(configuration);
	}
	
	public void setDefaultConfiguration(Configuration configuration) {
		this.defaultConfiguration = configuration;
		addConfiguration(configuration);
	}

	public Configuration getServerConfiguration() {
		return serverConfiguration;
	}

	public Configuration getDefaultConfiguration() {
		return defaultConfiguration;
	}
	
	@Override
	public void addProperty(String key, Object value) {
		if (logger.isWarnEnabled()) {
			logger.warn("Adding property with [{}]=[{}] to IN-MEMORY configuration - NOT PERSISTENT!", key, value);
		}
		
		super.addProperty(key, value);
	}
	
	@Override
	public void setProperty(String key, Object value) {
		logger.error("Adding properties on the *CompositeConfiguration* is disabled, since it also clears the values of the underlying database configurations");
		throw new NotImplementedException("Adding properties is disabled, since it also clears the values of the underlying database configurations");
	}
	
	public void clearCache() {
		if(getDefaultConfiguration() instanceof CachingDatabaseConfiguration) {
			((CachingDatabaseConfiguration)getDefaultConfiguration()).clearCache();
		}
		
		if(getServerConfiguration() instanceof CachingDatabaseConfiguration) {
			((CachingDatabaseConfiguration)getServerConfiguration()).clearCache();
		}
	}
}
