package org.sevensource.commons.jdbcconfiguration.impl.spring;

import javax.inject.Inject;
import javax.sql.DataSource;

import org.apache.commons.lang.StringUtils;
import org.sevensource.commons.jdbcconfiguration.ConfigurationException;
import org.sevensource.commons.jdbcconfiguration.database.ExtendedCompositeConfiguration;
import org.sevensource.commons.jdbcconfiguration.database.ExtendedDatabaseConfigurationFactory;
import org.sevensource.commons.jdbcconfiguration.impl.DefaultConfiguration;
import org.sevensource.commons.jdbcconfiguration.impl.DefaultConstants;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;


@Configuration
public abstract class AbstractDefaultConfigConfiguration<T extends DefaultConfiguration> {
	
	public final static String CONFIGURATIONS_DATASOURCE_NAME = "configurationsDataSource";
	
	
	@Bean(name="compositeDatabaseConfiguration")
	@Inject
	public ExtendedCompositeConfiguration compositeDatabaseConfiguration(Environment env) {
		DataSource dataSource = getDataSource();
		
		ExtendedDatabaseConfigurationFactory factory = databaseConfigurationFactory(dataSource, env);
		return factory.buildConfiguration();
	}
	

	@Bean(name="configuration")
	@Inject
	public T configuration(Environment env, ExtendedCompositeConfiguration compositeDatabaseConfiguration) {
		T configuration = getConfigurationInstance(env, compositeDatabaseConfiguration);
		configuration.setServerName(resolveServerName(env));
		configuration.setApplicationName(resolveApplicationName(env));
		
		return configuration;
	}
	
	
	protected String resolveServerName(Environment env) {
		final String name = env.getProperty(DefaultConstants.SERVER_NAME);
		if(StringUtils.isBlank(name)) {
			final String msg = String.format("Please specify a servername (Parameter [%s])", DefaultConstants.SERVER_NAME);
			throw new ConfigurationException(msg);
		}
		return name;
	}
	
	protected String resolveApplicationName(Environment env) {		
		final String name = env.getProperty(DefaultConstants.APPLICATION_NAME);
		if(StringUtils.isBlank(name)) {
			final String msg = String.format("Please specify a applicationname (Parameter [%s])", DefaultConstants.APPLICATION_NAME);
			throw new ConfigurationException(msg);
		}
		return name;
	}
	
	protected abstract DataSource getDataSource();
	protected abstract T getConfigurationInstance(Environment env, ExtendedCompositeConfiguration compositeDatabaseConfiguration);
	protected abstract ExtendedDatabaseConfigurationFactory databaseConfigurationFactory(DataSource dataSource, Environment env);
}
